from petisco.event.queue.domain.queue_config import QueueConfig
from petisco.event.queue.domain.specific_queue_config import SpecificQueueConfig


class QueueConfigMother:
    @staticmethod
    def default():
        return QueueConfig.default()

    @staticmethod
    def with_ttl_10():
        return QueueConfig.default(default_retry_ttl=10, default_main_ttl=10)

    @staticmethod
    def with_specific_queue_config(
        specific_queue_config: SpecificQueueConfig,
        default_retry_ttl: int,
        default_main_ttl: int,
    ):
        return QueueConfig(
            default_retry_ttl=default_retry_ttl,
            default_main_ttl=default_main_ttl,
            specific_queue_configs={"specific": specific_queue_config},
        )



from meiga import Error

from petisco.domain.value_objects.client_id import ClientId
from petisco.domain.value_objects.user_id import UserId
from petisco.domain.value_objects.uuid import Uuid


class ClientNotFoundError(Error):
    def __init__(self, client_id: ClientId):
        self.message = f"{self.__class__.__name__} [client_id: {client_id.value}]"


class ClientAlreadyExistError(Error):
    def __init__(self, client_id: ClientId):
        self.message = f"{self.__class__.__name__} [client_id: {client_id.value}]"


class UserNotFoundError(Error):
    def __init__(self, user_id: UserId):
        self.message = f"{self.__class__.__name__} [user_id: {user_id.value}]"


class UserAlreadyExistError(Error):
    def __init__(self, user_id: UserId):
        self.message = f"{self.__class__.__name__} [user_id: {user_id.value}]"


class EntityAlreadyExistError(Error):
    def __init__(self, repository_name: str, table_name: str, entity_id: Uuid):
        self.message = f"[Repository: {repository_name} | Table: {table_name} | entity_id: {entity_id.value}]"


class EntityNotFoundError(Error):
    def __init__(self, repository_name: str, entity_id: Uuid):
        self.message = f"[Repository: {repository_name} | entity_id: {entity_id.value}]"


class EntitiesNotFoundError(Error):
    def __init__(self, repository_name: str):
        self.message = f"[Repository: {repository_name}]"



from time import sleep

import pytest
from meiga import BoolResult, isSuccess

from petisco import Event, EventSubscriber
from petisco.fixtures.testing_decorators import testing_with_rabbitmq
from tests.modules.event.mothers.event_user_created_mother import EventUserCreatedMother
from tests.modules.event.mothers.rabbitmq_event_bus_mother import RabbitMqEventBusMother
from tests.modules.event.mothers.rabbitmq_event_configurer_mother import (
    RabbitMqEventConfigurerMother,
)
from tests.modules.event.mothers.rabbitmq_event_consumer_mother import (
    RabbitMqEventConsumerMother,
)
from tests.modules.event.spies.spy_events import SpyEvents


@pytest.mark.integration
@testing_with_rabbitmq
def test_should_publish_event_without_rabbitmq_configuration():
    event = EventUserCreatedMother.random()

    bus = RabbitMqEventBusMother.default()
    bus.publish(event)

    bus.configurer.clear()


@pytest.mark.integration
@testing_with_rabbitmq
def test_should_publish_event_without_rabbitmq_configuration_and_info_id():
    event = EventUserCreatedMother.random()

    bus = RabbitMqEventBusMother.with_info_id()
    bus.publish(event)
    bus.configurer.clear()


@pytest.mark.integration
@testing_with_rabbitmq
def test_should_publish_event_with_previous_rabbitmq_configuration():
    event = EventUserCreatedMother.random()

    configurer = RabbitMqEventConfigurerMother.default()
    configurer.configure_event(event)

    bus = RabbitMqEventBusMother.default()
    bus.publish(event)

    configurer.clear()


@pytest.mark.integration
@testing_with_rabbitmq
def test_should_publish_event_only_on_store_queue_with_previous_rabbitmq_configuration():
    event = EventUserCreatedMother.random()
    configurer = RabbitMqEventConfigurerMother.default()
    configurer.configure_event(event)

    bus = RabbitMqEventBusMother.default()
    bus.retry_publish_only_on_store_queue(event)

    configurer.clear()


@pytest.mark.integration
@testing_with_rabbitmq
def test_should_retry_publish_only_on_store_queue_not_affecting_default_event_queue():

    spy_consumer_default_queue = SpyEvents()
    spy_consumer_store = SpyEvents()

    def assert_consumer_default_queue(event: Event) -> BoolResult:
        spy_consumer_default_queue.append(event)
        return isSuccess

    def assert_consumer_store(event: Event) -> BoolResult:
        spy_consumer_store.append(event)
        return isSuccess

    event = EventUserCreatedMother.random()
    subscribers = [
        EventSubscriber(
            event_name=event.event_name,
            event_version=event.event_version,
            handlers=[assert_consumer_default_queue],
        )
    ]

    configurer = RabbitMqEventConfigurerMother.default()
    configurer.configure_subscribers(subscribers)

    bus = RabbitMqEventBusMother.default()
    bus.retry_publish_only_on_store_queue(event)

    consumer = RabbitMqEventConsumerMother.default()
    consumer.add_subscribers(subscribers)
    consumer.add_handler_on_store(assert_consumer_store)

    consumer.start()

    sleep(1.0)

    consumer.stop()
    configurer.clear()

    spy_consumer_default_queue.assert_number_unique_events(0)
    spy_consumer_default_queue.assert_count_by_event_id(event.event_id, 0)
    spy_consumer_store.assert_number_unique_events(1)
    spy_consumer_store.assert_first_event(event)
    spy_consumer_store.assert_count_by_event_id(event.event_id, 1)


@pytest.mark.integration
@testing_with_rabbitmq
def test_should_publish_and_then_consumer_retry_publish_only_on_store_queue_not_affecting_default_event_queue():

    spy_consumer_default_queue = SpyEvents()
    spy_consumer_store = SpyEvents()

    def assert_consumer_default_queue(event: Event) -> BoolResult:
        spy_consumer_default_queue.append(event)
        bus = RabbitMqEventBusMother.default()
        bus.retry_publish_only_on_store_queue(event)
        return isSuccess

    def assert_consumer_store(event: Event) -> BoolResult:
        spy_consumer_store.append(event)
        return isSuccess

    event = EventUserCreatedMother.random()
    subscribers = [
        EventSubscriber(
            event_name=event.event_name,
            event_version=event.event_version,
            handlers=[assert_consumer_default_queue],
        )
    ]

    configurer = RabbitMqEventConfigurerMother.default()
    configurer.configure_subscribers(subscribers)

    bus = RabbitMqEventBusMother.default()
    bus.publish(event)

    consumer = RabbitMqEventConsumerMother.default()
    consumer.add_subscribers(subscribers)
    consumer.add_handler_on_store(assert_consumer_store)

    consumer.start()

    sleep(1.0)

    consumer.stop()
    configurer.clear()

    spy_consumer_default_queue.assert_count_by_event_id(event.event_id, 1)
    spy_consumer_store.assert_count_by_event_id(event.event_id, 2)



import inspect
from typing import Dict, TypeVar, Callable

from deprecation import deprecated

from petisco.application.singleton import Singleton
from petisco.application.interface_repository import IRepository

Repository = TypeVar("Repository", bound=IRepository)


class Repositories(metaclass=Singleton):
    def __init__(self, repositories: Dict[str, Repository] = None):
        self._repositories = repositories if repositories else {}

    def get_info(self):
        return {
            name: repository.info() for name, repository in self._repositories.items()
        }

    @staticmethod
    def info():
        return Repositories.get_instance().get_info()

    @staticmethod
    def load(provider: Callable):
        repositories = provider()
        return Repositories(repositories)

    @staticmethod
    def get_instance():
        try:
            return Repositories()
        except Exception as e:  # noqa E722
            frame_info = inspect.stack()[1]
            raise ImportError(
                f"Repositories must be configured. If not, you cannot obtain repositories\n"
                f"Following code must be executed after Repositories initialization:\n"
                f"\tfilename: {frame_info.filename}\n"
                f"\tlineno: {frame_info.lineno}\n"
                f"\tfunction: {frame_info.function}\n"
                f"\tcode_context: {frame_info.code_context}\n\n"
            )

    def add(self, name: str, repository: Repository):
        if name in self._repositories:
            raise NameError(f"Repository {name} is already added to Repositories")
        self._repositories[name] = repository

    def remove(self, name: str):
        if name in self._repositories:
            del self._repositories[name]
        else:
            raise IndexError(f"Repository cannot be removed. {name} not exists")

    @staticmethod
    def get(name: str) -> Repository:
        repositories = Repositories.get_instance()._repositories
        if repositories is None:
            raise ValueError(
                "Repositories: no repository has been declared. Please, initialize it (i.e Repositories.from_provider(provider_func))"
            )
        repository = repositories.get(name)
        if not repository:
            raise ValueError(
                f"Repositories: {name} repository is not defined.  Please, add it (i.e Repositories.from_provider(provider_func))"
            )
        return repository

    @staticmethod
    @deprecated("This method is deprecated. Please, use Repositories.load")
    def from_provider(provider: Callable):
        return Repositories.load(provider)



from petisco.persistence.persistence import Persistence
from petisco.persistence.sql.mysql.mysql_database import MySqlDatabase


def mysql_is_running_locally() -> bool:
    try:
        database = MySqlDatabase.local_connection_checker()
        Persistence().add(database)
        Persistence().create()
        is_running_locally = True
    except:  # noqa: E722
        is_running_locally = False
    Persistence().remove("test", skip_if_not_exist=True)
    return is_running_locally



import inspect
import os
import time
from datetime import datetime
from distutils.util import strtobool
from os import environ
from typing import Callable, Dict, Any

from dataclasses import dataclass
from deprecation import deprecated

from petisco.application.deploy_checker import DeployChecker
from petisco.event.bus.infrastructure.not_implemented_event_bus import (
    NotImplementedEventBus,
)
from petisco.event.configurer.infrastructure.not_implemented_configurer import (
    NotImplementedEventConfigurer,
)
from petisco.event.consumer.infrastructure.not_implemented_event_comsumer import (
    NotImplementedEventConsumer,
)
from petisco.event.legacy.publisher.domain.interface_event_publisher import (
    IEventPublisher,
)
from petisco.event.shared.domain.config_events import ConfigEvents
from petisco.event.shared.domain.service_deployed import ServiceDeployed
from petisco.event.legacy.subscriber.domain.interface_event_subscriber import (
    IEventSubscriber,
)
from petisco.event.shared.domain.service_restarted import ServiceRestarted
from petisco.event.shared.infrastructure.configure_events_infrastructure import (
    configure_events_infrastructure,
)
from petisco.frameworks.interface_application import IApplication
from petisco.logger.interface_logger import INFO, ILogger, ERROR
from petisco.logger.log_message import LogMessage
from petisco.logger.not_implemented_logger import NotImplementedLogger
from petisco.notifier.infrastructure.not_implemented_notifier import (
    NotImplementedNotifier,
)
from petisco.notifier.domain.interface_notifier import INotifier
from petisco.notifier.domain.notifier_message import NotifierMessage
from petisco.application.config.config import Config
from petisco.application.singleton import Singleton
from petisco.application.interface_repository import IRepository
from petisco.application.interface_app_service import IService, IAppService
from petisco.tasks.infrastructure.apscheduler_task_executor import (
    APSchedulerTaskExecutor,
)
from petisco import __version__
from petisco.tasks.infrastructure.not_implemented_task_executor import (
    NotImplementedTaskExecutor,
)


@dataclass
class Petisco(metaclass=Singleton):
    _app_name: str
    _app_version: str
    _logger: ILogger = NotImplementedLogger()
    _notifier: INotifier = NotImplementedNotifier()
    application: IApplication = None
    services_provider: Callable = None
    repositories_provider: Callable = None
    _options: Dict[str, Any] = None
    info: Dict = None
    _persistence_models: Dict[str, Dict[str, Any]] = None
    persistence_sources: Dict[str, Dict] = None
    config: Config = None
    event_publisher: IEventPublisher = None
    event_subscriber: IEventSubscriber = None
    _environment: str = None
    repositories: Dict[str, Any] = None
    services: Dict[str, Any] = None

    def __init__(self, config: Config):
        self.config = config
        self._app_name = config.app_name
        self._app_version = config.app_version
        self._deploy_time = config.deploy_time
        self._logger = config.get_logger()
        self._environment = environ.get("ENVIRONMENT", None)
        self.info = {
            "app_name": self._app_name,
            "app_version": self._app_version,
            "deploy_time": self._deploy_time,
            "petisco_version": __version__,
            "environment": self._environment,
            "elapsed_time": {},
        }
        self._notifier = config.get_notifier()
        self._set_persistence()
        self._set_events_configuration()
        self._set_tasks()
        self._options = config.options
        self._deploy_checker = DeployChecker(
            deploy_time=self._deploy_time,
            courtesy_minutes=int(os.getenv("PETISCO_DEPLOY_COURTESY_MINUTES", 60)),
        )

    @staticmethod
    def get_instance():
        try:
            return Petisco()
        except Exception as e:  # noqa E722
            frame_info = inspect.stack()[1]
            raise ImportError(
                f"Petisco must be configured when the application starts. \n"
                f"Petisco has been requested before its initial configuration.\n "
                f"Following code must be executed after Petisco initialization:\n"
                f"\tfilename: {frame_info.filename}\n"
                f"\tlineno: {frame_info.lineno}\n"
                f"\tfunction: {frame_info.function}\n"
                f"\tcode_context: {frame_info.code_context}\n\n"
                f"\t Possible Solutions:\n"
                f"\t  * If you are testing and need any preconfigured Petisco, you can use the fixture given_any_petisco.\n"
                f"\t  * If you are testing and need specific preconfigured Petisco, you can create a fixture loading a petisco and then clearing.\n"
                f"\t  * If you are running a Petisco application, please review your loading process, petisco.yml is probably failing.\n"
            )

    @staticmethod
    def from_filename(filename: str):
        """
        Parameters
        ----------
        filename
            YAML-based configuration file (default petisco.yml)
        """
        print(f"Loading petisco from: {filename}")

        config = Config.from_filename(filename).unwrap_or_throw()

        return Petisco(config=config)

    def configure_events(self, filename: str):
        """
        Parameters
        ----------
        filename
            YAML-based event management configuration file (default petisco.events.yml)
        """

        message_broker_type = os.environ.get("PETISCO_EVENT_MESSAGE_BROKER")
        if message_broker_type and message_broker_type == "notimplemented":
            return

        config_events = ConfigEvents.from_filename(filename).unwrap_or_throw()
        (
            self.event_bus,
            self.event_configurer,
            self.event_consumer,
        ) = configure_events_infrastructure(config_events, self._logger)

        if config_events.event_subscribers:
            clear_subscriber_before = strtobool(
                os.getenv(
                    "PETISCO_LEGACY_RABBITMQ_CONFIGURER_CLEAR_SUBSCRIBER_BEFORE",
                    "false",
                )
            )
            clear_store_before = strtobool(
                os.getenv(
                    "PETISCO_LEGACY_RABBITMQ_CONFIGURER_CLEAR_STORE_BEFORE", "false"
                )
            )

            self.event_configurer.configure_subscribers(
                config_events.event_subscribers,
                clear_subscriber_before=clear_subscriber_before,
                clear_store_before=clear_store_before,
            )
            self.event_consumer.add_subscribers(config_events.event_subscribers)
        else:
            self.event_configurer.configure()

        if config_events.store_queue_subscriber:
            self.event_consumer.add_handler_on_store(
                config_events.store_queue_subscriber
            )

        if config_events.queues_subscribers:
            for queue, handlers in config_events.queues_subscribers.items():
                for handler in handlers:
                    self.event_consumer.add_handler_on_queue(queue, handler)

        self.config_events = config_events

        self.info["config_events"] = self.config_events.info()

    def _publish_deploy_event(self):
        if self._deploy_checker.was_recently_deployed(datetime.utcnow()):
            event = ServiceDeployed(
                app_name=self._app_name, app_version=self._app_version
            )
        else:
            event = ServiceRestarted(
                app_name=self._app_name, app_version=self._app_version
            )

        if self.config_events and self.config_events.publish_deploy_event:
            self.event_bus.publish(event)

        # Legacy Approach
        if self.config.config_events.publish_deploy_event:
            self.event_publisher.publish(event)

    def _notify_restart(self):
        if not self._deploy_checker.was_recently_deployed(datetime.utcnow()):
            self._notifier.publish(
                NotifierMessage(
                    title="Service Restarted",
                    message=f"{self._app_name} has been restarted. Original deploy was at {self._deploy_checker.get_deploy_time()}",
                    info_petisco=self.get_info(),
                )
            )

    def _set_tasks(self):
        config_tasks = self.config.config_tasks
        if config_tasks and config_tasks.tasks:
            self.info["tasks"] = {}
            for task_name, config_task in config_tasks.tasks.items():
                self.info["tasks"][task_name] = config_task.to_dict()
            self.task_executor = APSchedulerTaskExecutor()
        else:
            self.task_executor = NotImplementedTaskExecutor()

    def _schedule_tasks(self):
        config_tasks = self.config.config_tasks
        if config_tasks.tasks:
            self.task_executor.start(config_tasks)

    def _unschedule_tasks(self):
        config_tasks = self.config.config_tasks
        if config_tasks.tasks:
            self.task_executor.stop()

    def _set_persistence(self):
        self._persistence_models = {}
        self.persistence_sources = {}
        config_persistence = self.config.config_persistence
        if config_persistence and config_persistence.configs:
            for config_key, config_value in config_persistence.configs.items():
                if config_value.type == "sql":
                    import_database_models_func = (
                        config_persistence.get_import_database_models_func(config_key)
                    )

                    config_value.config(import_database_models_func)

                    self._persistence_models[
                        config_key
                    ] = config_persistence.get_models(config_key)
                else:
                    config_value.config()
                self.persistence_sources[config_key] = {
                    "configured": True,
                    "type": config_value.type,
                }

    def _set_services_and_repositories_from_providers(self):
        config_providers = self.config.config_providers
        if not config_providers:
            return
        if config_providers.config_dependencies:
            config_providers.config_dependencies()

        if config_providers.services_provider:
            self.services = self._load_services(config_providers.services_provider)

            info_services = {}
            for key, service in self.services.items():
                if hasattr(service, "info"):
                    info_services[key] = service.info()
                else:
                    raise TypeError(
                        f"Service with key {key} ({type(service)}) must implement info"
                    )
            self.info["services"] = info_services
        if config_providers.repositories_provider:
            self.repositories = self._load_repositories(
                config_providers.repositories_provider
            )
            info_repositories = {}
            for key, repository in self.repositories.items():
                if hasattr(repository, "info"):
                    info_repositories[key] = repository.info()
                else:
                    raise TypeError(
                        f"Repository with key {key} ({type(repository)}) must implement info"
                    )

            self.info["repositories"] = info_repositories

    def _set_events_configuration(self):
        self.event_bus = NotImplementedEventBus()
        self.event_configurer = NotImplementedEventConfigurer()
        self.event_consumer = NotImplementedEventConsumer()
        self.config_events = None

        self._legacy_set_events_configuration()

    def _legacy_set_events_configuration(self):
        config_events = self.config.config_events
        if not config_events:
            return

        if config_events.config_event_publisher.provider:
            self.event_publisher = config_events.config_event_publisher.provider()
            if hasattr(self.event_publisher, "info"):
                self.info["event_publisher"] = self.event_publisher.info()
            else:
                raise TypeError(
                    f"Given event_publisher ({type(self.event_publisher)}) must implement info"
                )

        if config_events.config_event_subscriber.provider:
            subscribers = config_events.config_event_subscriber.subscribers
            self.event_subscriber = config_events.config_event_subscriber.provider(
                subscribers
            )
            if hasattr(self.event_subscriber, "info"):
                self.info["event_subscriber"] = self.event_subscriber.info()
            else:
                raise TypeError(
                    f"Given event_subscriber ({type(self.event_subscriber)}) must implement info"
                )

    def _log_status(self):
        if self.info:
            self._logger.log(INFO, LogMessage(data={"message": {"info": self.info}}))

        if self._options:
            self._logger.log(
                INFO, LogMessage(data={"message": {"options": self._options}})
            )

    def _start(self):
        self._set_services_and_repositories_from_providers()
        self.event_subscriber.start()
        self.event_consumer.start()
        self._schedule_tasks()
        self._log_status()
        self._announce()

    def _announce(self):
        try:
            self._publish_deploy_event()
            self._notify_restart()
        except Exception as exc:
            log_message = LogMessage(layer="petisco")
            log_message.set_message(
                f"Error announcing alice-petisco application {str(exc)}"
            )
            self._logger.log(ERROR, log_message)

    def load_services_and_repositories(self):
        self._set_services_and_repositories_from_providers()

    def start(self):
        self._start()
        self._application.start()

    def get_app(self):
        self._start()
        self._application = self.config.get_application()
        return self._application.get_app()

    def get_application(self):
        return self._application

    def stop(self):
        self.event_subscriber.stop()
        self.event_consumer.stop()
        self._unschedule_tasks()

    def _load_repositories(self, repositories_provider: Callable):
        start_time = time.time()
        repositories = repositories_provider()
        elapsed_time = time.time() - start_time
        self.info["elapsed_time"][
            "load_repositories"
        ] = f"{int(elapsed_time * 1000.0)} ms"
        return repositories

    def _load_services(self, services_provider: Callable):
        start_time = time.time()
        services = services_provider()
        elapsed_time = time.time() - start_time
        self.info["elapsed_time"]["load_services"] = f"{int(elapsed_time * 1000.0)} ms"
        return services

    @staticmethod
    def get_event_bus():
        return Petisco.get_instance().event_bus

    @staticmethod
    def get_event_consumer():
        return Petisco.get_instance().event_consumer

    @staticmethod
    def get_logger():
        return Petisco.get_instance()._logger

    @staticmethod
    def get_notifier():
        return Petisco.get_instance()._notifier

    @staticmethod
    def get_app_name():
        return Petisco.get_instance()._app_name

    @staticmethod
    def get_app_version():
        return Petisco.get_instance()._app_version

    @staticmethod
    def get_environment():
        return Petisco.get_instance()._environment

    @staticmethod
    def get_info():
        return Petisco.get_instance().info

    # TODO Remove deprecated options
    @staticmethod
    @deprecated("This method is deprecated. Please, use AppServices.get()")
    def get_app_service(key: str) -> IAppService:
        services = Petisco.get_instance().services
        if not services:
            raise ValueError(
                "Petisco: no service has been declared. Please, add it to petisco.yml."
            )
        service = services.get(key)
        if not service:
            raise ValueError(
                f"Petisco: {key} service is not defined. Please, add it to petisco.yml"
            )
        return service

    @staticmethod
    @deprecated("This method is deprecated. Please, use AppServices.get()")
    def get_service(key: str) -> IService:
        return Petisco.get_app_service(key)

    @staticmethod
    @deprecated("This method is deprecated. Please, use Repositories.get()")
    def get_repository(key: str) -> IRepository:
        repositories = Petisco.get_instance().repositories
        if repositories is None:
            raise ValueError(
                "Petisco: no repository has been declared. Please, add it to petisco.yml"
            )
        repository = repositories.get(key)
        if not repository:
            raise ValueError(
                f"Petisco: {key} repository is not defined. Please, add it to petisco.yml"
            )
        return repository

    @staticmethod
    def persistence_models(persistence_entry: str) -> Dict[str, str]:
        persistence_models = {}
        try:
            persistence_models = Petisco.get_instance()._persistence_models[
                persistence_entry
            ]
        except:  # noqa E722
            pass
        return persistence_models

    @staticmethod
    def get_persistence_model(persistence_entry: str, key: str) -> Any:
        return Petisco.persistence_models(persistence_entry).get(key)

    @staticmethod
    def persistence_mongodb_client():
        from petisco.persistence.pymongo.pymongo_persistence import PyMongoPersistence

        return PyMongoPersistence().client

    @staticmethod
    @deprecated("This method is deprecated.")
    def get_event_publisher():
        return Petisco.get_instance().event_publisher

    @staticmethod
    @deprecated("This method is deprecated.")
    def get_event_subscriber():
        return Petisco.get_instance().event_subscriber



import pytest

from petisco import IRepository, ClientId


@pytest.mark.unit
def test_should_inherit_from_repository():
    class Repository(IRepository):
        pass

    repository = Repository()

    assert repository.info() == {"name": "Repository"}


@pytest.mark.unit
def test_should_inherit_from_repository_and_overwrite_info_method():
    class Repository(IRepository):
        def info(self):
            return None

    repository = Repository()

    assert repository.info() is None


@pytest.mark.unit
def test_should_inherit_from_repository_and_configure_one_with_client_id():
    class Repository(IRepository):
        pass

    repository = Repository()

    assert not hasattr(repository, "client_id")

    repository_with_client_id = repository.with_client_id(ClientId("acme"))

    assert hasattr(repository_with_client_id, "client_id")
    assert repository_with_client_id is not repository


@pytest.mark.unit
def test_should_inherit_from_repository_and_configure_one_with_client_id_getting_client_id():
    class Repository(IRepository):
        pass

    repository = Repository()
    repository_with_client_id = repository.with_client_id(ClientId("acme"))

    client_id = repository_with_client_id.get_client_id()
    assert isinstance(client_id, ClientId)

    client_id_value = repository_with_client_id.get_client_id_value()
    assert isinstance(client_id_value, str)


@pytest.mark.unit
def test_should_inherit_from_repository_and_fail_when_getting_client_id_without_configuring_it():
    class Repository(IRepository):
        pass

    repository = Repository()
    with pytest.raises(AttributeError) as excinfo:
        _ = repository.get_client_id()
        assert (
            "Repository needs client_id. Please, Repository.use with_client_id() to get a valid"
            in str(excinfo.value)
        )

    with pytest.raises(AttributeError) as excinfo:
        _ = repository.get_client_id_value()
        assert (
            "Repository needs client_id. Please, Repository.use with_client_id() to get a valid"
            in str(excinfo.value)
        )



from petisco.event.chaos.domain.interface_event_chaos import IEventChaos


class NotImplementedEventChaos(IEventChaos):
    def info(self):
        return {
            "EVENT_CHAOS_PERCENTAGE_SIMULATE_NACK": "NOT_IMPLEMENTED",
            "EVENT_CHAOS_DELAY_AFTER_OPERATION_SECONDS": "NOT_IMPLEMENTED",
            "EVENT_CHAOS_PERCENTAGE_SIMULATE_FAILURES": "NOT_IMPLEMENTED",
        }

    def nack_simulation(self, *args, **kwargs) -> bool:
        return False

    def failure_simulation(self, *args, **kwargs) -> bool:
        return False

    def delay(self):
        pass



import pytest
from petisco import EventId
from tests.modules.event.unit.test_event import UserCreated


@pytest.mark.unit
def test_should_create_an_event_from_deprecated_version():

    deprecated_body = {
        "name": "Name",
        "event_version": "2.0.0",
        "event_id": "e4e1d",
        "event_name": "UserCreated",
        "event_occurred_on": "2020-04-30 09:05:58.965449",
    }
    expected_body = {
        "data": {
            "id": "e4e1d".rjust(EventId.length(), "0"),  # Refilled with zeros
            "type": "user.created",
            "version": "0",  # old format event will be set to 0 version
            "occurred_on": "2020-04-30 09:05:58.965449",
            "attributes": {"name": "Name"},
            "type_message": "domain_event",
            "meta": {},
        }
    }

    retrieved_event = UserCreated.from_deprecated_dict(deprecated_body)

    assert retrieved_event.to_dict() == expected_body


@pytest.mark.unit
@pytest.mark.parametrize(
    "input_info_id,expected_info_id",
    [
        (
            {
                "client_id": "client",
                "user_id": "mwy-yQWxt2O0ECnw",
                "correlation_id": "889bd",
            },
            {
                "client_id": "client",
                "user_id": "mwy-yQWxt2O0ECnw",
                "correlation_id": "889bd",
                "ip": None,
            },
        ),
        (
            {"user_id": "mwy-yQWxt2O0ECnw", "correlation_id": "889bd"},
            {
                "client_id": None,
                "user_id": "mwy-yQWxt2O0ECnw",
                "correlation_id": "889bd",
                "ip": None,
            },
        ),
        (
            {
                "client_id": "client",
                "user_id": "mwy-yQWxt2O0ECnw",
                "correlation_id": "889bd",
                "ip": "0.0.0.0",
            },
            {
                "client_id": "client",
                "user_id": "mwy-yQWxt2O0ECnw",
                "correlation_id": "889bd",
                "ip": "0.0.0.0",
            },
        ),
    ],
)
def test_should_create_an_event_from_deprecated_version_with_info_id(
    input_info_id, expected_info_id
):

    deprecated_body = {
        "document_id": "SIgradfXfWNbtDfsA0sqwgJWpMs=",
        "manual": False,
        "event_version": "2.0.0",
        "event_id": "57062",
        "event_name": "DocumentBackRequested",
        "event_occurred_on": "2020-05-08 12:17:15.162161",
        "info_id": input_info_id,
    }
    expected_body = {
        "data": {
            "id": "57062".rjust(EventId.length(), "0"),  # Refilled with zeros
            "type": "document.back.requested",
            "version": "0",
            "occurred_on": "2020-05-08 12:17:15.162161",
            "attributes": {
                "document_id": "SIgradfXfWNbtDfsA0sqwgJWpMs=",
                "manual": False,
            },
            "type_message": "domain_event",
            "meta": {"info_id": expected_info_id},
        }
    }

    retrieved_event = UserCreated.from_deprecated_dict(deprecated_body)

    assert retrieved_event.to_dict() == expected_body



import connexion
from flask_bcrypt import Bcrypt

from flask_cors import CORS

from petisco.frameworks.flask.application.json_encoder import JSONEncoder
from petisco.frameworks.interface_application import IApplication


class FlaskApplication(IApplication):
    def __init__(
        self,
        application_name: str,
        swagger_dir: str,
        config_file: str,
        port: int = 8080,
    ):
        self.application_name = application_name
        self.app = connexion.App(__name__, specification_dir=swagger_dir)
        self.config_file = config_file
        self.port = port
        self._configure()

    def _configure(self):
        CORS(self.app.app)
        self.app.app.config["BCRYPT_HANDLE_LONG_PASSWORDS"] = True
        self.bcrypt = Bcrypt(self.app.app)
        self.app.app.json_encoder = JSONEncoder
        self.app.add_api(self.config_file, arguments={"title": self.application_name})

    def start(self):
        self.app.run(port=self.port)

    def get_config(self):
        return self.config

    def get_app(self):
        return self.app

    config = property(get_config)



from time import sleep

import pytest
from meiga import isSuccess, isFailure

from petisco import (
    Event,
    subscriber_handler,
    RabbitMQEventPublisher,
    RabbitMQEventSubscriber,
    ConfigEventSubscriber,
    RabbitMqConnector,
)
from petisco.fixtures import testing_with_rabbitmq
from tests.fixtures import TrackedEventsSpy


def await_for_events():
    sleep(1.5)


@pytest.mark.integration
@testing_with_rabbitmq
def test_should_work_successfully_a_happy_path_pub_sub(
    given_any_petisco,
    make_user_created_event,
    given_random_organization,
    given_random_service,
    given_random_topic,
):
    event_1 = make_user_created_event()
    event_2 = make_user_created_event()

    tracked_events_spy = TrackedEventsSpy()

    @subscriber_handler()
    def main_handler(event: Event):
        tracked_events_spy.append(event)
        return isSuccess

    publisher = RabbitMQEventPublisher(
        connector=RabbitMqConnector(),
        organization=given_random_organization,
        service=given_random_service,
        topic=given_random_topic,
    )

    publisher.publish_events([event_1, event_2])

    subscriber = RabbitMQEventSubscriber(
        connector=RabbitMqConnector(),
        subscribers={
            "auth": ConfigEventSubscriber(
                organization=given_random_organization,
                service=given_random_service,
                topic=given_random_topic,
                handler=main_handler,
            )
        },
    )
    subscriber.start()

    await_for_events()

    tracked_events_spy.assert_number_events(2)

    subscriber.stop()


@pytest.mark.integration
@testing_with_rabbitmq
def test_should_publish_reject_and_requeue_from_dead_letter_exchange(
    given_any_petisco,
    make_user_created_event,
    given_random_organization,
    given_random_service,
    given_random_topic,
    given_a_short_message_ttl,
):
    event = make_user_created_event()
    tracked_events_spy = TrackedEventsSpy()
    tracked_requeue_events_dead_letter_spy = TrackedEventsSpy()

    publisher = RabbitMQEventPublisher(
        connector=RabbitMqConnector(),
        organization=given_random_organization,
        service=given_random_service,
        topic=given_random_topic,
    )

    @subscriber_handler()
    def main_handler(event: Event):
        tracked_events_spy.append(event)
        return isFailure

    @subscriber_handler()
    def requeue_from_dead_letter(event: Event):
        tracked_requeue_events_dead_letter_spy.append(event)
        publisher.publish(event)
        return isSuccess

    publisher.publish(event)

    subscriber = RabbitMQEventSubscriber(
        connector=RabbitMqConnector(),
        subscribers={
            "auth": ConfigEventSubscriber(
                organization=given_random_organization,
                service=given_random_service,
                topic=given_random_topic,
                handler=main_handler,
            ),
            "dead-letter": ConfigEventSubscriber(
                organization=given_random_organization,
                service=given_random_service,
                topic=given_random_topic,
                handler=requeue_from_dead_letter,
                dead_letter=True,
            ),
        },
    )
    subscriber.start()

    await_for_events()

    tracked_events_spy.assert_number_events(1)
    tracked_requeue_events_dead_letter_spy.assert_number_events(1)

    subscriber.stop()


@pytest.mark.integration
@testing_with_rabbitmq
def test_should_work_successfully_a_happy_path_pub_sub_with_subscribers_simulations_nack_everything(
    given_any_petisco,
    make_user_created_event,
    given_random_organization,
    given_random_service,
    given_random_topic,
):
    event = make_user_created_event()
    tracked_events_spy = TrackedEventsSpy()

    @subscriber_handler(percentage_simulate_nack=1.0)
    def main_handler_everything_nack(event: Event):
        tracked_events_spy.append(event)
        return isSuccess

    publisher = RabbitMQEventPublisher(
        connector=RabbitMqConnector(),
        organization=given_random_organization,
        service=given_random_service,
        topic=given_random_topic,
    )

    publisher.publish_events([event, event])

    subscriber = RabbitMQEventSubscriber(
        connector=RabbitMqConnector(),
        subscribers={
            "auth": ConfigEventSubscriber(
                organization=given_random_organization,
                service=given_random_service,
                topic=given_random_topic,
                handler=main_handler_everything_nack,
            )
        },
    )
    subscriber.start()

    await_for_events()

    tracked_events_spy.assert_number_events(0)

    subscriber.stop()



from time import sleep

import pytest

from petisco.frameworks.flask.flask_extension_is_installed import (
    flask_extension_is_installed,
)
from petisco.persistence.sqlalchemy.sqlalchemy_extension_is_installed import (
    sqlalchemy_extension_is_installed,
)


@pytest.mark.end2end
@pytest.mark.skipif(
    not flask_extension_is_installed(), reason="Flask extension is not installed"
)
@pytest.mark.skipif(
    not sqlalchemy_extension_is_installed(),
    reason="SQLAlchemy extension is not installed",
)
def test_should_return_200_when_create_an_user(
    client_app, given_auth_token_headers_creator, given_any_name
):
    headers = given_auth_token_headers_creator(type_token="ADMIN_TOKEN")

    response = client_app.post(
        "/petisco/user", headers=headers, data=dict(name=given_any_name.value)
    )
    assert response.status_code == 200

    sleep(1.0)

    response = client_app.get("/petisco/users/count", headers=headers)
    assert response.status_code == 200
    assert response.json == {"users_count": 1}


@pytest.mark.end2end
@pytest.mark.skipif(
    not flask_extension_is_installed(), reason="Flask extension is not installed"
)
@pytest.mark.skipif(
    not sqlalchemy_extension_is_installed(),
    reason="SQLAlchemy extension is not installed",
)
def test_should_return_409_when_call_create_user_with_invalid_name(
    client_app, given_auth_token_headers_creator, given_code_injection_name
):

    headers = given_auth_token_headers_creator(type_token="ADMIN_TOKEN")

    response = client_app.post(
        "/petisco/user", headers=headers, data=dict(name=given_code_injection_name)
    )
    assert response.status_code == 409



import json
from datetime import datetime
from typing import Optional

import validators

from petisco.domain.date_parser import DateParser
from petisco.webhooks.webhook.domain.secret import Secret
from petisco.domain.aggregate_roots.aggregate_root import AggregateRoot
from petisco.webhooks.webhook.domain.invalid_url_error import InvalidUrlError
from petisco.webhooks.webhook.domain.signature_algorithm import SignatureAlgorithm
from petisco.webhooks.webhook.domain.webhook_created import WebhookCreated
from petisco.webhooks.webhook.domain.webhook_id import WebhookId


class Webhook(AggregateRoot):
    @staticmethod
    def create(
        active: bool,
        post_url: str,
        api_key: str,
        secret: Secret,
        event_name: str,
        event_version: Optional[str] = "1",
        algorithm: Optional[SignatureAlgorithm] = SignatureAlgorithm.sha256(),
    ):
        webhook_id = WebhookId.generate()

        webhook = Webhook(
            webhook_id=webhook_id,
            active=active,
            post_url=post_url,
            api_key=api_key,
            secret=secret,
            event_name=event_name,
            event_version=event_version,
            algorithm=algorithm,
        )
        webhook.record(
            WebhookCreated(
                webhook_id=webhook_id,
                active=active,
                subscribed_event_name=event_name,
                subscribed_event_version=event_version,
            )
        )
        return webhook

    @staticmethod
    def from_dict(kdict: dict):
        return Webhook(
            webhook_id=WebhookId(kdict.get("webhook_id")),
            active=kdict.get("active"),
            post_url=kdict.get("post_url"),
            api_key=kdict.get("api_key"),
            secret=Secret(kdict.get("secret")),
            event_name=kdict.get("event_name"),
            event_version=kdict.get("event_version"),
            algorithm=SignatureAlgorithm(kdict.get("algorithm")),
            created_at=DateParser.datetime_from_str(kdict.get("created_at")),
            updated_at=DateParser.datetime_from_str(kdict.get("updated_at")),
        )

    def to_dict(self):
        return {
            "webhook_id": self.webhook_id.value,
            "active": self.active,
            "post_url": self.post_url,
            "api_key": self.api_key,
            "secret": self.secret.value,
            "event_name": self.event_name,
            "event_version": self.event_version,
            "algorithm": self.algorithm.value,
            "created_at": DateParser.str_from_datetime(self.created_at),
            "updated_at": DateParser.str_from_datetime(self.updated_at),
        }

    def __init__(
        self,
        webhook_id: WebhookId,
        active: bool,
        post_url: str,
        api_key: str,
        secret: Secret,
        event_name: str,
        event_version: Optional[str] = "1",
        algorithm: Optional[SignatureAlgorithm] = SignatureAlgorithm.sha256(),
        created_at: Optional[datetime] = datetime.utcnow(),
        updated_at: Optional[datetime] = None,
    ):
        self.webhook_id = webhook_id
        self.active = active
        self.post_url = post_url
        self.api_key = api_key
        self.secret = secret
        self.event_name = event_name
        self.event_version = event_version
        self.algorithm = algorithm
        self.created_at = created_at
        self.updated_at = updated_at
        self.validate()
        super().__init__()

    def __repr__(self):
        return json.dumps(self.to_dict())

    def __eq__(self, other):
        if issubclass(other.__class__, self.__class__) or issubclass(
            self.__class__, other.__class__
        ):
            return self.to_dict() == other.to_dict()
        else:
            return False

    def validate(self):
        validation_result = validators.url(self.post_url)

        if not validation_result:
            raise InvalidUrlError()



import pytest

from petisco.event.queue.domain.specific_queue_config import SpecificQueueConfig


@pytest.mark.unit
@pytest.mark.parametrize(
    "queue_name,wildcard,expected_result",
    [
        ("petisco.service.1.event.user_created", "*send_webhook*", False),
        (
            "petisco.service.1.event.user_created.send_webhook_on_user_created",
            "*send_webhook*",
            True,
        ),
        (
            "petisco.service.1.event.user_created.send_webhook_on_user_created",
            "send_webhook*",
            False,
        ),
    ],
)
def test_should_queue_config_works_as_expected(queue_name, wildcard, expected_result):

    specific_queue_config = SpecificQueueConfig(wildcard=wildcard)

    assert specific_queue_config.has_specific_config(queue_name) == expected_result


@pytest.mark.unit
def test_should_queue_config_works_as_expected_with_specific_parameters():
    specific_queue_config = SpecificQueueConfig(
        wildcard="*send_webhook*", specific_retry_ttl=10, specific_main_ttl=20
    )
    assert specific_queue_config.get_retry_ttl() == 10
    assert specific_queue_config.get_main_ttl() == 20


@pytest.mark.unit
@pytest.mark.parametrize(
    "queue_name,queue_config_dict,expected_result",
    [
        (
            "petisco.service.1.event.user_created",
            {
                "when_queue_contains": "*send_webhook*",
                "use": {"retry_ttl": 1, "main_ttl": 1},
            },
            False,
        ),
        (
            "petisco.service.1.event.user_created.send_webhook_on_user_created",
            {
                "when_queue_contains": "*send_webhook*",
                "use": {"retry_ttl": 1, "main_ttl": 1},
            },
            True,
        ),
        (
            "petisco.service.1.event.user_created.send_webhook_on_user_created",
            {"when_queue_contains": "*send_webhook*", "use": {"retry_ttl": 1}},
            True,
        ),
    ],
)
def test_should_queue_config_works_as_expected_when_load_from_dict(
    queue_name, queue_config_dict, expected_result
):

    specific_queue_config = SpecificQueueConfig.from_dict(
        queue_config_dict, default_retry_ttl=10, default_main_ttl=20
    )

    assert specific_queue_config.has_specific_config(queue_name) == expected_result



import pytest
import requests_mock
from meiga.assertions import assert_success

from petisco.webhooks.webhook.domain.webhook_result import WebhookResult
from tests.modules.webook.mothers.webhook_mother import WebhookMother, DEFAULT_POST_URL
from tests.modules.webook.mothers.webhook_sender_mother import WebhookSenderMother


@pytest.mark.integration
def test_should_be_failure_when_ping_a_webhook_and_url_is_not_reachable():
    webhook = WebhookMother.default()
    sender = WebhookSenderMother.default()

    result = sender.ping(webhook)

    assert_success(result, value_is_instance_of=WebhookResult)

    webhook_result = result.value
    assert not webhook_result.is_success


@pytest.mark.integration
def test_should_be_failure_when_execute_a_webhook_and_url_is_not_reachable():
    webhook = WebhookMother.default()
    sender = WebhookSenderMother.default()

    result = sender.execute(webhook, {"payload": "ok"})

    assert_success(result, value_is_instance_of=WebhookResult)

    webhook_result = result.value
    assert not webhook_result.is_success


@pytest.mark.integration
def test_should_be_success_when_ping_a_webhook_and_url_is_reachable_by_mocking_it():
    webhook = WebhookMother.default()
    sender = WebhookSenderMother.default()

    with requests_mock.Mocker() as m:
        m.post(DEFAULT_POST_URL)
        result = sender.ping(webhook)
        webhook_result = result.value
        assert webhook_result.is_success


@pytest.mark.integration
def test_should_be_success_when_execute_a_webhook_and_url_is_reachable_by_mocking_it():
    webhook = WebhookMother.default()
    sender = WebhookSenderMother.default()

    with requests_mock.Mocker() as m:
        m.post(DEFAULT_POST_URL)
        result = sender.execute(webhook, {"payload": "ok"})
        webhook_result = result.value
        assert webhook_result.is_success



from petisco import Event, UserId
from tests.modules.event.mothers.defaults import DEFAULT_USER_ID


class UserUpdated(Event):
    user_id: UserId

    def __init__(self, user_id: UserId):
        self.user_id = user_id
        super().__init__()


class EventUserUpdatedMother:
    @staticmethod
    def random():
        return UserUpdated(user_id=UserId.generate())

    @staticmethod
    def default():
        return UserUpdated(user_id=DEFAULT_USER_ID)

    @staticmethod
    def with_user_id(user_id: UserId):
        return UserUpdated(user_id=user_id)



from time import sleep

import pytest

from petisco.event.legacy.publisher.infrastructure.rabbitmq_event_publisher import (
    RabbitMQEventPublisher,
)
from petisco.event.shared.infrastructure.rabbitmq.rabbitmq_connector import (
    RabbitMqConnector,
)
from petisco.event.shared.infrastructure.rabbitmq.rabbitmq_is_running_locally import (
    rabbitmq_is_running_locally,
)


@pytest.mark.integration
@pytest.mark.skipif(
    not rabbitmq_is_running_locally(), reason="RabbitMQ is not running locally"
)
def test_should_create_a_rabbitmq_event_publisher_and_publish_a_event(
    make_user_created_event,
):
    rabbitmq_connector = RabbitMqConnector()
    event = make_user_created_event()

    publisher = RabbitMQEventPublisher(
        connector=rabbitmq_connector,
        organization="acme",
        service="service",
        topic="service-event",
    )

    publisher.publish(event)


@pytest.mark.integration
def test_should_fail_publisher_when_connection_parameter_are_not_valid():
    with pytest.raises(TypeError):
        _ = RabbitMQEventPublisher(
            connector=None,
            organization="acme",
            service="service",
            topic="service-event",
        )


@pytest.mark.integration
@pytest.mark.skipif(
    not rabbitmq_is_running_locally(), reason="RabbitMQ is not running locally"
)
def test_should_create_a_rabbitmq_event_publisher_and_publish_a_event_after_heartbeat(
    make_user_created_event,
):
    event = make_user_created_event()
    rabbitmq_connector = RabbitMqConnector()
    rabbitmq_connector.heartbeat = 2

    publisher = RabbitMQEventPublisher(
        connector=rabbitmq_connector,
        organization="acme",
        service="service",
        topic="service-event",
    )

    sleep(10)  # wait more than the heartbeat

    publisher.publish(event)



import os
import traceback
from typing import Dict, Any, List

import yaml
from meiga import Result, Error, Failure, Success
from meiga.decorators import meiga
from yaml.parser import ParserError, ScannerError

from petisco.application.config.config_persistence import ConfigPersistence
from petisco.application.config.config_providers import ConfigProviders
from petisco.notifier.config.config_notifier import ConfigNotifier
from petisco.notifier.domain.interface_notifier import INotifier
from petisco.notifier.infrastructure.not_implemented_notifier import (
    NotImplementedNotifier,
)
from petisco.tasks.config.config_tasks import ConfigTasks
from petisco.frameworks.interface_application import IApplication
from petisco.logger.interface_logger import ILogger
from petisco.logger.logging_based_logger import LoggingBasedLogger
from petisco.application.config.events.config_events import ConfigEvents
from petisco.application.config.config_file_not_found_error import (
    ConfigFileNotFoundError,
)
from petisco.application.config.config_file_not_valid_error import (
    ConfigFileNotValidError,
)
from petisco.application.config.config_framework import ConfigFramework
from petisco.application.config.config_logger import ConfigLogger
from petisco.logger.not_implemented_logger import NotImplementedLogger


class Config:
    def __init__(
        self,
        app_name: str,
        app_version: str,
        deploy_time: str,
        petisco_yml_folder: str = None,
        config_tasks: ConfigTasks = None,
        config_framework: ConfigFramework = None,
        config_logger: ConfigLogger = None,
        config_notifier: ConfigNotifier = None,
        config_persistence: ConfigPersistence = None,
        config_providers: ConfigProviders = None,
        config_events: ConfigEvents = None,
        options: Dict = None,
    ):
        self.app_name = app_name
        self.app_version = app_version
        self.deploy_time = deploy_time
        self.petisco_yml_folder = petisco_yml_folder
        self.config_tasks = config_tasks
        self.config_framework = config_framework
        self.config_logger = config_logger
        self.config_notifier = config_notifier
        self.config_persistence = config_persistence
        self.config_providers = config_providers
        self.config_events = config_events
        self.options = options

    @staticmethod
    @meiga
    def from_filename(filename: str) -> Result[Any, Error]:
        if not os.path.isfile(filename):
            return Failure(ConfigFileNotFoundError(filename))
        try:
            petisco_yml_folder = os.path.dirname(filename)
            with open(filename) as file:
                petisco_dict = yaml.load(file, Loader=yaml.FullLoader)
                petisco_dict["petisco_yml_folder"] = petisco_yml_folder
                config = Config.from_dict(petisco_dict).unwrap_or_return()
                return Success(config)
        except (ParserError, ScannerError) as e:
            message = f"Error loading {filename} file: {repr(e.__class__)} {e} | {traceback.format_exc()}"
            return Failure(ConfigFileNotValidError(message))

    @staticmethod
    @meiga
    def from_dict(yaml_dict: Dict) -> Result[Any, Error]:
        petisco_yml_folder = yaml_dict.get("petisco_yml_folder")
        app_config = yaml_dict.get("app")
        app_name = app_config.get("name")

        app_version = Config.get_version(
            petisco_yml_folder, app_config.get("version")
        ).unwrap_or_return()

        deploy_time = Config.get_deploy_time(
            petisco_yml_folder, app_config.get("deploy_time")
        ).unwrap_or_return()

        config_tasks = ConfigTasks.from_dict(yaml_dict.get("tasks"))

        config_framework = ConfigFramework.from_dict(yaml_dict.get("framework"))
        config_logger = Config.get_config_logger(
            yaml_dict.get("logger")
        ).unwrap_or_return()

        config_notifier = ConfigNotifier.from_dict(yaml_dict.get("notifier"))

        config_persistence = ConfigPersistence.from_dict(yaml_dict.get("persistence"))

        providers = yaml_dict.get("providers")
        config_providers = (
            ConfigProviders.from_dict(providers) if providers else ConfigProviders()
        )

        config_events = ConfigEvents.from_dict(yaml_dict.get("events"))

        options = app_config.get("options")

        return Success(
            Config(
                petisco_yml_folder=petisco_yml_folder,
                app_name=app_name,
                app_version=app_version,
                deploy_time=deploy_time,
                config_tasks=config_tasks,
                config_framework=config_framework,
                config_logger=config_logger,
                config_notifier=config_notifier,
                config_providers=config_providers,
                config_events=config_events,
                config_persistence=config_persistence,
                options=options,
            )
        )

    @staticmethod
    def get_version(petisco_yml_folder, config_version) -> Result[str, Error]:
        if isinstance(config_version, str):
            return Success(config_version)
        else:
            version_filename = (
                config_version.get("from_file", "VERSION")
                if config_version
                else "VERSION"
            )
            try:

                version_filename = f"{petisco_yml_folder}/{version_filename}"
                if not os.path.isfile(version_filename):
                    return Failure(
                        ConfigFileNotValidError(
                            f"VERSION file does not exist ({version_filename})"
                        )
                    )
                version = open(version_filename, "r").read()
            except:  # noqa
                return Failure(
                    ConfigFileNotValidError(
                        "If you don't specify the version directly, you must associate a file with from_file key"
                    )
                )

            return Success(version.rstrip("\n"))

    @staticmethod
    def get_deploy_time(petisco_yml_folder, config_deploy_time) -> Result[str, Error]:
        if isinstance(config_deploy_time, str):
            return Success(config_deploy_time)
        else:
            deploy_time_filename = (
                config_deploy_time.get("from_file", "DEPLOY")
                if config_deploy_time
                else "DEPLOY"
            )

            try:
                deploy_time_filename = f"{petisco_yml_folder}/{deploy_time_filename}"
                if not os.path.isfile(deploy_time_filename):
                    return Failure(
                        ConfigFileNotValidError(
                            f"DEPLOY file does not exist ({deploy_time_filename})"
                        )
                    )
                version = open(deploy_time_filename, "r").read()
            except:  # noqa
                return Failure(
                    ConfigFileNotValidError(
                        "If you don't specify the deploy_time directly, you must associate a file with from_file key or have available a DEPLOY file in the root path"
                    )
                )
            return Success(version.rstrip("\n"))

    @staticmethod
    def get_config_logger(config_logger_dict: Dict) -> Result[ConfigLogger, Error]:

        if not config_logger_dict:
            return Success(ConfigLogger())
        else:
            return Success(ConfigLogger.from_dict(config_logger_dict))

    def get_logger(self) -> ILogger:
        if not self.config_logger or self.config_logger.selected_logger != "logging":
            return NotImplementedLogger()
        else:
            return LoggingBasedLogger(
                logger_name=self.config_logger.name,
                format=self.config_logger.format,
                config=self.config_logger.config,
            )

    def get_notifier(self) -> INotifier:
        if not self.config_notifier:
            return NotImplementedNotifier()
        return self.config_notifier.provider()

    def get_application(self) -> IApplication:
        if not self.config_framework:
            return None

        if self.config_framework.selected_framework != "flask":
            raise TypeError(
                f"Selected framework ({self.config_framework.selected_framework}) is not allowed. Try with flask."
            )
        else:
            if not self.config_framework.port_env:
                port = self.config_framework.port
            else:
                port = (
                    self.config_framework.port
                    if not os.environ.get(self.config_framework.port_env)
                    else os.environ.get(self.config_framework.port_env)
                )

            from petisco.frameworks.flask.application.flask_application import (
                FlaskApplication,
            )

            conf_file_split: List = self.config_framework.config_file.rsplit("/", 1)

            if len(conf_file_split) == 1:
                swagger_dir = self.petisco_yml_folder
                config_file = conf_file_split[0]
            else:
                swagger_dir, config_file = conf_file_split

            return FlaskApplication(
                application_name=self.app_name,
                swagger_dir=swagger_dir,
                config_file=config_file,
                port=port,
            )
